/*
	==============================================================================================
								Math lib vectors in 2D
								Created by faceH0r 29.05.2005
	==============================================================================================
*/

#ifndef _VECTOR_2D
#define _VECTOR_2D

// ===============================================================================================

	#include <math.h>

// ===============================================================================================

class c_vec_2d{

public:

	// Variables class vector in 2d
	float x;
	float y;

	// Methods
	c_vec_2d(void);		// Default init
	c_vec_2d(const float x,const float y);	// Special methid need

	// Functions 
	void vecSet(const float x,const float y);	// Set vector position
	void vecZero(void);		// Set is NULL vector

	float vecLenght();		// Calculate vector lenght
	float vecDot(const c_vec_2d &v1);	// Calculate DOT
	float vecCosDot(const c_vec_2d &v1);	// Calculate Cos(this,v2)

	c_vec_2d& vecReflection(const c_vec_2d &v);		// Calculate reflect
	c_vec_2d& vecNormalize();		// Equ for -1.0f to 1.0f vector
	c_vec_2d& vecInvert();			// Invert all coordinates
	c_vec_2d& vecRotate(const float angle);		// Rotate vector

	c_vec_2d vecNormal();			// Calculate normal

	// Special reserved symbols
	c_vec_2d operator+(const c_vec_2d &v);	// Add vectors
	c_vec_2d operator-(const c_vec_2d &v);	// Substract vectors
	c_vec_2d operator*(const c_vec_2d &v);	// Mulriply vectors
	c_vec_2d operator/(const c_vec_2d &v);	// Division vectors

	c_vec_2d& operator+=(const c_vec_2d &v);	// Add current vector
	c_vec_2d& operator-=(const c_vec_2d &v);	// Substract current vector
	c_vec_2d& operator*=(const c_vec_2d &v);	// Multiply current vector
	c_vec_2d& operator/=(const c_vec_2d &v);	// Division current vector

	c_vec_2d operator*(const float a);		// Multiple vector for number
	c_vec_2d operator/(const float a);		// Division vector for number

	c_vec_2d& operator*=(const float a);	// Multiple current vector for number
	c_vec_2d& operator/=(const float a);	// Division current vector for number

	bool operator==(const c_vec_2d &v);	// Return true if vectors equal
	bool operator!=(const c_vec_2d &v);	// Return true if vectors is not equal

private:

};

// ===============================================================================================

c_vec_2d::c_vec_2d(void){
	this->x=this->y=0.0f;
}

// ===============================================================================================

c_vec_2d::c_vec_2d(const float x,const float y){
	this->x=x;
	this->y=y;
}

// ===============================================================================================

void c_vec_2d::vecSet(const float x,const float y){
	this->x=x;
	this->y=y;
}

// ===============================================================================================

void c_vec_2d::vecZero(void){
	this->x=this->y=0.0f;
}

// ===============================================================================================

float c_vec_2d::vecLenght(){
	return (float)sqrt(this->x*this->x+this->y*this->y);
}

// ===============================================================================================

float c_vec_2d::vecDot(const c_vec_2d &v1){
	return (float)(v1.x*this->x+v1.y*this->y);
}

// ===============================================================================================

float c_vec_2d::vecCosDot(const c_vec_2d &v1){
	float v_len1,v_len2;

	v_len1=sqrt(v1.x*v1.x+v1.y*v1.y);
	v_len2=sqrt(this->x*this->x+this->y*this->y);

	if(!v_len1)
		v_len1=1.0f;
	if(!v_len2)
		v_len2=1.0f;

	return (float)(v1.x*this->x+v1.y*this->y)/(v_len1*v_len2);
}

// ===============================================================================================

c_vec_2d& c_vec_2d::vecReflection(const c_vec_2d &n){
	float f_Cos=this->vecCosDot(n);

	this->x=this->x-2*f_Cos*n.x;
	this->y=this->y-2*f_Cos*n.y;

	return *this;
}

// ===============================================================================================

c_vec_2d& c_vec_2d::vecNormalize(){
	float vec_len;

	vec_len=1.0f/sqrt(this->x*this->x+this->y*this->y);

	this->x*=vec_len;
	this->y*=vec_len;

	return *this;
}

// ===============================================================================================

c_vec_2d& c_vec_2d::vecInvert(){
	this->x*=(-1);
	this->y*=(-1);

	return *this;
}

// ===============================================================================================

c_vec_2d& c_vec_2d::vecRotate(const float angle){
	float x_s,y_s;

	x_s=this->x*cos(angle)-this->y*sin(angle);
	y_s=this->x*sin(angle)+this->y*cos(angle);

	this->x=x_s;
	this->y=y_s;

	return *this;
}

// ===============================================================================================

c_vec_2d c_vec_2d::vecNormal(){
	float vec_len;

	vec_len=sqrt(this->x*this->x+this->y*this->y);

	if(!vec_len)
		vec_len=1.0f;

	return c_vec_2d(this->y/vec_len,-this->x/vec_len);
}

// ===============================================================================================

c_vec_2d c_vec_2d::operator+(const c_vec_2d &v){
	return c_vec_2d(this->x+v.x,this->y+v.y);
}

// ===============================================================================================

c_vec_2d c_vec_2d::operator-(const c_vec_2d &v){
	return c_vec_2d(this->x-v.x,this->y-v.y);
}

// ===============================================================================================

c_vec_2d c_vec_2d::operator*(const c_vec_2d &v){
	return c_vec_2d(this->x*v.x,this->y*v.y);
}

// ===============================================================================================

c_vec_2d c_vec_2d::operator/(const c_vec_2d &v){
	return c_vec_2d(this->x/v.x,this->y/v.y);
}

// ===============================================================================================

c_vec_2d c_vec_2d::operator*(const float a){
	return c_vec_2d(this->x*a,this->y*a);
}

// ===============================================================================================

c_vec_2d c_vec_2d::operator/(const float a){
	return c_vec_2d(this->x/a,this->y/a);
}

// ===============================================================================================

c_vec_2d& c_vec_2d::operator+=(const c_vec_2d &v){
	this->x+=v.x;
	this->y+=v.y;

	return *this;
}

// ===============================================================================================

c_vec_2d& c_vec_2d::operator-=(const c_vec_2d &v){
	this->x-=v.x;
	this->y-=v.y;

	return *this;
}

// ===============================================================================================

c_vec_2d& c_vec_2d::operator*=(const c_vec_2d &v){
	this->x*=v.x;
	this->y*=v.y;

	return *this;
}

// ===============================================================================================

c_vec_2d& c_vec_2d::operator/=(const c_vec_2d &v){
	this->x/=v.x;
	this->y/=v.y;

	return *this;
}

// ===============================================================================================

c_vec_2d& c_vec_2d::operator*=(const float a){
	this->x*=a;
	this->y*=a;

	return *this;
}

// ===============================================================================================

c_vec_2d& c_vec_2d::operator/=(const float a){
	float b=1.0f/a;

	this->x*=b;
	this->y*=b;

	return *this;
}

// ===============================================================================================

bool c_vec_2d::operator==(const c_vec_2d &v){
	return (this->x==v.x)&&(this->y==v.y);
}

// ===============================================================================================

bool c_vec_2d::operator!=(const c_vec_2d &v){
	return !((this->x==v.x)&&(this->y==v.y));
}

// ===============================================================================================

#endif _VECTOR_2D